﻿/* sra 5 v. 1.2
 * 
 * (c) Philippe Truillet
 * Modified: 06/01/2013
 * Last modified: 28/10/2014
 */

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

//using Microsoft.Speech.AudioFormat;
//using Microsoft.Speech.Recognition;
using System.Speech.Recognition;
using System.Speech.Recognition.SrgsGrammar;
using System.IO;

// bus ivy
using IvyBus;

// Ajout du support ivy 

/* sra using Microsoft Speech Platform */
namespace sra
{
    /// <summary>
    /// Logique d'interaction pour Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        // whether to use the command and control grammar or the dictation grammar
        // by default: dictation mode, any grammar
        
        SpeechRecognitionEngine _speechRecognizer;
        private Ivy myBus;
        private String ivy_address = "127.255.255.255:2010";
        private bool parse = false;
        private String grammar_file = "grammar.grxml";
        private Grammar commandGrammar;
        
        private String language = "fr-FR";
        
        private String[] args = null;
        private static int NP = 0;

        public Window1()
        {
            InitializeComponent();
            args = App.mArgs;
            this.WindowState = WindowState.Minimized;

            if (args != null)
            {
                // gère les arguments
                for (int i = 0; i < this.args.Length; i = i + 2)
                {
                    if (args[i].CompareTo("-b") == 0)
                    {
                        ivy_address = args[i+1];
                    }

                    if (args[i].CompareTo("-p") == 0)
                    {
                        // off by default
                        if (args[i + 1].CompareTo("on") == 0)
                        {
                            parse = true;
                        }

                    }
                    if (args[i].CompareTo("-g") == 0)
                    {
                                               
                        grammar_file = args[i+1];

                    }
                    if (args[i].CompareTo("-l") == 0)
                    {
                                               
                        language = args[i+1];

                    }
                }
            }
            
            // ivy
            myBus = new Ivy("sra5", "sra5 is ready");
            myBus.Start(ivy_address);
            int regexp_id = myBus.BindMsg("^sra5 -(.*) (.*)", receive);

            _speechRecognizer = new SpeechRecognitionEngine(new CultureInfo(language));

            commandGrammar = new Grammar(@grammar_file);
            commandGrammar.Name = "grammar";
            commandGrammar.Enabled = true;

            //receives input that does not match any of its loaded and enabled Grammar objects.
            _speechRecognizer.LoadGrammarCompleted += new EventHandler<LoadGrammarCompletedEventArgs>(_speechRecognizerLoadGrammarCompleted);
            _speechRecognizer.LoadGrammarAsync(commandGrammar);

            // Affichage de la grammaire utilisée
            grammar_textBox.Clear();
            grammar_textBox.AppendText(@grammar_file);

            // speech detected
            _speechRecognizer.SpeechDetected += new EventHandler<SpeechDetectedEventArgs>(_speechRecognizer_SpeechDetected);
            _speechRecognizer.SpeechRecognized += new EventHandler<SpeechRecognizedEventArgs>(_speechRecognizer_SpeechRecognized);
            _speechRecognizer.SpeechRecognitionRejected += new EventHandler<SpeechRecognitionRejectedEventArgs>(_speechRecognizerSpeechRecognitionRejected);
            // inutilisé
            //_speechRecognizer.SpeechHypothesized += new EventHandler<SpeechHypothesizedEventArgs>(_speechRecognizerSpeechHypothesized);

            _speechRecognizer.SetInputToDefaultAudioDevice();
            _speechRecognizer.RecognizeAsync(RecognizeMode.Multiple);
            _speechRecognizer.MaxAlternates = 4;
             // TRACE();
        }
        
        // reçoit des ordres ivy
        /*
         * Fonction qui reçoit un message ivy (change de grammaire)
         * 
         * 
         */
        private void receive(object sender, IvyMessageEventArgs e)
        {
            String[] args = e.GetArguments();

            String argument = args[0];

            if (argument.CompareTo("g") == 0) // changement de grammaire
            {
                // Console.Write(args[0]);
                _speechRecognizer.UnloadGrammar(commandGrammar);
                grammar_file = args[1];
                commandGrammar = new Grammar(@grammar_file);
                commandGrammar.Name = "grammar";
                commandGrammar.Enabled = true;
                _speechRecognizer.LoadGrammarAsync(commandGrammar);
                grammar_textBox.Clear();
                grammar_textBox.AppendText(@grammar_file);
            }
            else if (argument.CompareTo("p") == 0) // changement de parsage
            {
                if (args[1].CompareTo("on") == 0)
                {
                    parse = true;
                }
                else
                {
                    parse = false;
                }
            }

        }

        /* Fonctions de traitement de GUI */
        protected void TRACE()
        {
            result_textBox.Clear();
            result_textBox.AppendText(args.Length + " ARGUMENTS\n");         
            
            for (int i = 0; i < args.Length; i++)
            {
                result_textBox.AppendText("\n" + args[i]);
            }             
        }

        private void Button_Exit(object sender, RoutedEventArgs e)
        {
            if (myBus != null)
                myBus.Stop();
            if (_speechRecognizer != null)
            {
                _speechRecognizer.UnloadAllGrammars();
                // _speechRecognizer.Dispose();
            }
            Close();
        }

        protected override void OnClosing(System.ComponentModel.CancelEventArgs e)
        {
            if (myBus !=null)
                myBus.Stop();

            if (_speechRecognizer != null)
            {
                _speechRecognizer.UnloadAllGrammars();
                _speechRecognizer.Dispose();
            }
        }

        /* == Handlers ==*/
        /* Parole détectée */
        void _speechRecognizer_SpeechDetected(object sender, SpeechDetectedEventArgs e)
        {
            // Rien pour l'instant 
            // Console.Write("\rSpeech Detected: \n");
        }

        /* Reconnaissance vocale */
        void _speechRecognizer_SpeechRecognized(object sender, SpeechRecognizedEventArgs e)
        {
            /* afficher le résultat dans la fenêtre */
            result_textBox.Clear();
            result_textBox.AppendText("Recognized: " + e.Result.Text + "\nConfidence: " + e.Result.Confidence +  "\n Semantics: " + e.Result.Semantics.Value.ToString());
           
            if (this.parse == false)
            {
                myBus.SendMsg("sra5 Text=" + e.Result.Text + " Confidence=" + e.Result.Confidence);
            }
            else
            {
                myBus.SendMsg("sra5 Parsed=" + e.Result.Semantics.Value.ToString() + " Confidence=" + e.Result.Confidence + " NP=" + NP + " Num_A=0");
                NP++;
            }           
        }
        
        /* Parole rejetée */
        void _speechRecognizerSpeechRecognitionRejected(object sender, SpeechRecognitionRejectedEventArgs e)
        {
            // Console.Write("\rSpeech Rejected: \t{0} \n", e.Result.Text);
            myBus.SendMsg("sra5 Event=Speech_Rejected"); 
        }

        /* Reconnaissance vocale avec hypothèses */
        void _speechRecognizerSpeechHypothesized(object sender, SpeechHypothesizedEventArgs e)
        {
            // inutilisé ...
            /*
            String sem = "";
            if (e.Result.Semantics.Value != null)
                sem = e.Result.Semantics.Value.ToString();

            result_textBox.Clear();
            //result_textBox.AppendText(e.Result.Text + " " + e.Result.Confidence);
            result_textBox.AppendText(e.Result.Text + " " + e.Result.Confidence + "\n Semantics: " + sem);
            if (e.Result.Confidence > 0.70)
            {
                if (this.parse == false)
                {
                    myBus.SendMsg("sra Text=" + e.Result.Text);
                }
                else
                {
                    // myBus.SendMsg("sra Parsed=" + e.Result.Semantics + " Confidence=" + e.Result.Confidence + " NP=" + NP+ " Num_A=0");
                    myBus.SendMsg("sra Parsed=" + e.Result.Text + " Confidence=" + e.Result.Confidence + " NP=" + NP + " Num_A=0");
                    NP++;
                }
            }
             */
        }

        /* chargement de la grammaire */
        void _speechRecognizerLoadGrammarCompleted(object sender, LoadGrammarCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show(e.Error.Message);
                myBus.SendMsg("sra5 Event=Grammar_Error");
            }
            else
                myBus.SendMsg("sra5 Event=Grammar_Loaded"); 
        }
    }
}
